import os
import re
import sys
import json
import argparse
import tqdm
import pandas as pd
from functools import partial
from collections import OrderedDict
from youtube_transcript_api import YouTubeTranscriptApi
from utils import download_with_url, makedirs, parallel_run

def download_captions(channels):
    base_dir="./datasets/"
    c = channels
    print(c)
    video_id =[]
    text = []
    start = []
    duration = []
    names=[]
    file_list = os.listdir(base_dir+c+"/audio/")
    file_list_wav = [file for file in file_list if file.endswith(".wav")]
    # print(file_list_wav)
    for f in tqdm.tqdm(file_list_wav):
        try:
            video = f.split(".wav")[0]
            subtitle = YouTubeTranscriptApi.get_transcript(video)
            # print(subtitle)
            # break
            for s in range(len(subtitle)-1):
                # print(s)
                video_id.append(video)
                name = base_dir+c+'/audio/'+video+'.'+str(s).zfill(4)+'.wav'
                names.append(name)
                # print(subtitle[s]['text'])
                subtitle[s]['text'] = ''.join([c for c in subtitle[s]['text'] if c not in ('!', '?', ',', '.', '\n', '~', '"', "'")])
                # print(subtitle[s])
                text.append(subtitle[s]['text'])
                start.append(subtitle[s]['start'])
                duration.append(subtitle[s+1]['start'] - subtitle[s]['start'])
        except:
            # print(e)
            pass

    print(len(text), len(start), len(duration), len(names))
    df = pd.DataFrame({"id":video_id, "text":text, "start":start, "duration":duration, "name":names})
    makedirs(base_dir+c+'/text')
    df.to_csv(base_dir+c+'/text/subtitle.csv', encoding='utf-8')
    file_data=OrderedDict()
    for i in range(df.shape[0]):
        file_data[df['name'][i]]=df['text'][i]
    with open (base_dir+c+'/alignment.json', 'w', encoding="utf-8") as make_file:
        json.dump(file_data, make_file, ensure_ascii=False, indent="\n")
    print(c+' channel was finished')
        
def download_caption_batch(channels):
    print(channels)
    download_captions(channels)
    # fn = partial(download_captions)
    #
    # parallel_run(fn, channels,
    #         desc="Download caption", parallel=False)
        
if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # parser.add_argument('--base_dir', required=True)
    parser.add_argument('--channels', required=True)
    
    config = parser.parse_args()

    download_caption_batch(
            config.channels
    )



#-*- coding: utf-8 -*- 
import os
import re
import sys
import json
import requests
import subprocess
from tqdm import tqdm
from contextlib import closing
from multiprocessing import Pool
from collections import namedtuple
from datetime import datetime, timedelta
from shutil import copyfile as copy_file

PARAMS_NAME = "params.json"

class ValueWindow():
    def __init__(self, window_size=100):
        self._window_size = window_size
        self._values = []

    def append(self, x):
        self._values = self._values[-(self._window_size - 1):] + [x]

    @property
    def sum(self):
        return sum(self._values)

    @property
    def count(self):
        return len(self._values)

    @property
    def average(self):
        return self.sum / max(1, self.count)

    def reset(self):
        self._values = []

def prepare_dirs(config, hparams):
    if hasattr(config, "data_paths"):
        config.datasets = [
                os.path.basename(data_path) for data_path in config.data_paths]
        dataset_desc = "+".join(config.datasets)

    if config.load_path:
        config.model_dir = config.load_path
    else:
        config.model_name = "{}_{}".format(dataset_desc, get_time())
        config.model_dir = os.path.join(config.log_dir, config.model_name)

        for path in [config.log_dir, config.model_dir]:
            if not os.path.exists(path):
                os.makedirs(path)

    if config.load_path:
        load_hparams(hparams, config.model_dir)
    else:
        setattr(hparams, "num_speakers", len(config.datasets))

        save_hparams(config.model_dir, hparams)
        copy_file("hparams.py", os.path.join(config.model_dir, "hparams.py"))

def makedirs(path):
    if not os.path.exists(path):
        print(" [*] Make directories : {}".format(path))
        os.makedirs(path)

def remove_file(path):
    if os.path.exists(path):
        print(" [*] Removed: {}".format(path))
        os.remove(path)

def backup_file(path):
    root, ext = os.path.splitext(path)
    new_path = "{}.backup_{}{}".format(root, get_time(), ext)

    os.rename(path, new_path)
    print(" [*] {} has backup: {}".format(path, new_path))

def get_time():
    return datetime.now().strftime("%Y-%m-%d_%H-%M-%S")

def write_json(path, data):
    with open(path, 'w',encoding='utf-8') as f:
        json.dump(data, f, indent=4, sort_keys=True, ensure_ascii=False)

def load_json(path, as_class=False, encoding='utf-8'):
    with open(path,encoding=encoding) as f:
        content = f.read()
        content = re.sub(",\s*}", "}", content)
        content = re.sub(",\s*]", "]", content)

        if as_class:
            data = json.loads(content, object_hook=\
                    lambda data: namedtuple('Data', data.keys())(*data.values()))
        else:
            data = json.loads(content)
    #print(data)
    return data

def save_hparams(model_dir, hparams):
    param_path = os.path.join(model_dir, PARAMS_NAME)

    info = eval(hparams.to_json(). \
            replace('true', 'True').replace('false', 'False'))
    write_json(param_path, info)

    print(" [*] MODEL dir: {}".format(model_dir))
    print(" [*] PARAM path: {}".format(param_path))

def load_hparams(hparams, load_path, skip_list=[]):
    path = os.path.join(load_path, PARAMS_NAME)

    new_hparams = load_json(path)
    hparams_keys = vars(hparams).keys()

    for key, value in new_hparams.items():
        if key in skip_list or key not in hparams_keys:
            print("Skip {} because it not exists".format(key))
            continue

        if key not in ['job_name', 'num_workers', 'display', 'is_train', 'load_path'] or \
                key == "pointer_load_path":
            original_value = getattr(hparams, key)
            if original_value != value:
                print("UPDATE {}: {} -> {}".format(key, getattr(hparams, key), value))
                setattr(hparams, key, value)

def add_prefix(path, prefix):
    dir_path, filename = os.path.dirname(path), os.path.basename(path)
    return "{}/{}.{}".format(dir_path, prefix, filename)

def add_postfix(path, postfix):
    path_without_ext, ext = path.rsplit('.', 1)
    return "{}.{}.{}".format(path_without_ext, postfix, ext)

def remove_postfix(path):
    items = path.rsplit('.', 2)
    return items[0] + "." + items[2]

def parallel_run(fn, items, desc="", parallel=True):
    results = []

    if parallel:
        with closing(Pool()) as pool:
            for out in tqdm(pool.imap_unordered(
                    fn, items), total=len(items), desc=desc):
                if out is not None:
                    results.append(out)
    else:
        for item in tqdm(items, total=len(items), desc=desc):
            out = fn(item)
            if out is not None:
                results.append(out)

    return results

def which(program):
    if os.name == "nt" and not program.endswith(".exe"):
        program += ".exe"

    envdir_list = [os.curdir] + os.environ["PATH"].split(os.pathsep)

    for envdir in envdir_list:
        program_path = os.path.join(envdir, program)
        if os.path.isfile(program_path) and os.access(program_path, os.X_OK):
            return program_path

def get_encoder_name():
    if which("avconv"):
        return "avconv"
    elif which("ffmpeg"):
        return "ffmpeg"
    else:
        return "ffmpeg"

def download_with_url(url, dest_path, chunk_size=32*1024):
    with open(dest_path, "wb") as f:
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))

        for chunk in response.iter_content(chunk_size):
            if chunk: # filter out keep-alive new chunks
                f.write(chunk)
    return True

def str2bool(v):
    return v.lower() in ('true', '1')

def get_git_revision_hash():
    return subprocess.check_output(['git', 'rev-parse', 'HEAD']).decode("utf-8")

def get_git_diff():
    return subprocess.check_output(['git', 'diff']).decode("utf-8")

def warning(msg):
    print("="*40)
    print(" [!] {}".format(msg))
    print("="*40)
    print()

def query_yes_no(question, default=None):
    # Code from https://stackoverflow.com/a/3041990
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")



import os 
import matplotlib
from jamo import h2j, j2hcj
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
font_path = 'C:/Windows/Fonts/NanumGothic.ttf'
fontprop = fm.FontProperties(fname=font_path)
matplotlib.use('Agg')
#matplotlib.rc('font', family=font_name)
from text import PAD, EOS
from utils import add_postfix
from text.korean import normalize

def plot(alignment, info, text, isKorean=True):
    char_len, audio_len = alignment.shape # 145, 200

    fig, ax = plt.subplots(figsize=(char_len/5, 5))
    im = ax.imshow(
            alignment.T,
            aspect='auto',
            origin='lower',
            interpolation='none')

    xlabel = 'Encoder timestep'
    ylabel = 'Decoder timestep'

    if info is not None:
        xlabel += '\n{}'.format(info)

    plt.xlabel(xlabel)
    plt.ylabel(ylabel)

    if text:
        if isKorean:
            jamo_text = j2hcj(h2j(normalize(text)))
        else:
            jamo_text=text
        pad = [PAD] * (char_len - len(jamo_text) - 1)

        plt.xticks(range(char_len),
                [tok for tok in jamo_text] + [EOS] + pad, fontproperties=fontprop)

    if text is not None:
        while True:
            if text[-1] in [EOS, PAD]:
                text = text[:-1]
            else:
                break
        plt.title(text, fontproperties=fontprop)

    plt.tight_layout()

def plot_alignment(
        alignment, path, info=None, text=None, isKorean=True):

    if text:
        tmp_alignment = alignment[:len(h2j(text)) + 2]

        plot(tmp_alignment, info, text, isKorean)
        plt.savefig(path, format='png')
    else:
        plot(alignment, info, text, isKorean)
        plt.savefig(path, format='png')

    print(" [*] Plot saved: {}".format(path))



#-*- coding: utf-8 -*- 
import os
import re
import sys
import csv
import json
import argparse
import tqdm
import youtube_dl
import pandas as pd
from glob import glob
from pydub import AudioSegment
from functools import partial
from collections import OrderedDict
from youtube_transcript_api import YouTubeTranscriptApi
from utils import makedirs, parallel_run

class ACCP:
    def __init__(self, output_dir:str, youtube_url:str) -> None:
        self.output_dir = output_dir
        self.youtube_url = youtube_url
    
    def download_audio(self) -> None:
        base_dir="./datasets/"
        download_path = os.path.join(base_dir+self.output_dir+"/wavs", '%(id)s.%(ext)s')

        # youtube_dl options
        ydl_opts = {
            'format': 'bestaudio/best',
            'postprocessors':[{
                'key':'FFmpegExtractAudio',
                'preferredcodec': 'wav',
                'preferredquality': '192'
            }],
            'postprocessors_args':[
                '-ar', '21000'
            ],
            'prefer_ffmpeg':True,
            'keepvideo':False,
            'outtmpl': download_path # 다운로드 경로 설정
        }

        try:
            with youtube_dl.YoutubeDL(ydl_opts) as ydl:
                ydl.download([self.youtube_url])
        except Exception as e:
            print('error', e)

    def download_captions(self) -> None:
        base_dir="./datasets/"
        c = self.output_dir
        video_id =[]
        text = []
        start = []
        duration = []
        names=[]
        full_names=[]
        file_list = os.listdir(base_dir+c+"/wavs/")
        file_list_wav = [file for file in file_list if file.endswith(".wav")]
        for f in tqdm.tqdm(file_list_wav):
            try:
                video = f.split(".wav")[0]
                subtitle = YouTubeTranscriptApi.get_transcript(video, languages=['ko'])
                for s in range(len(subtitle)-1):
                    video_id.append(video)
                    full_name = base_dir+c+'/wavs/'+video+'.'+str(s).zfill(4)+'.wav'
                    full_names.append(full_name)
                    name = video+'.'+str(s).zfill(4)+'.wav'
                    names.append(name)
                    subtitle[s]['text'] = ''.join([c for c in subtitle[s]['text'] if c not in ('!', '?', ',', '.', '\n', '~', '"', "'")])
                    text.append(subtitle[s]['text'])
                    start.append(subtitle[s]['start'])
                    duration.append(subtitle[s+1]['start'] - subtitle[s]['start'])
            except:
                pass

        df = pd.DataFrame({"id":video_id, "text":text, "start":start, "duration":duration, "name":full_names})
        makedirs(base_dir+c+'/text')
        df.to_csv(base_dir+c+'/text/subtitle.csv', encoding='utf-8')
        res = [i +'|'+ j for i, j in zip(names, text)] 
        df2 = pd.DataFrame({"name":res})
        df2.to_csv(base_dir+c+'/metadata.csv', encoding='utf-8', header=False, index=False)
        file_data=OrderedDict()
        for i in range(df.shape[0]):
            file_data[df['name'][i]]=df['text'][i]
        with open (base_dir+c+'/alignment.json', 'w', encoding="utf-8") as make_file:
            json.dump(file_data, make_file, ensure_ascii=False, indent="\n")
        print(c+' channel was finished')

    def audio_split(self) -> None:
        base_dir="./datasets/"+self.output_dir+'/wavs/*.wav'
        audio_paths = glob(base_dir)
        audio_paths.sort()
        fn = partial(split_with_caption, 'wav')
        parallel_run(fn, audio_paths, desc="Split with caption", parallel=False)



def split_with_caption(audio_path, skip_idx=0, out_ext="wav") -> list:
    df = pd.read_csv(audio_path.split('wavs')[0]+'text/subtitle.csv')
    filename = os.path.basename(audio_path).split('.', 1)[0]

    audio = read_audio(audio_path)
    df2 = df[df['id'].apply(str)==filename]
    df2['end']=round((df2['start']+df2['duration'])*1000).astype(int)
    df2['start'] = round(df2['start']*1000).astype(int)
    edges = df2[['start', 'end']].values.tolist()
    
    audio_paths = []
    for idx, (start_idx, end_idx) in enumerate(edges[skip_idx:]):
        start_idx = max(0, start_idx)
        

        target_audio_path = "{}/{}.{:04d}.{}".format(
                os.path.dirname(audio_path), filename, idx, out_ext)

        segment=audio[start_idx:end_idx]
        
        segment.export(target_audio_path, "wav")  # for soundsegment

        audio_paths.append(target_audio_path)

    return audio_paths

def read_audio(audio_path):
    return AudioSegment.from_file(audio_path)



            



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="accp", # Replace with your own username
    version="0.0.1",
    author="Seugnhun Jeong",
    author_email="zldzmfoq12@naver.com",
    description="A pakage for crawling and processing audio, caption from Youtube",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zldzmfoq12/aud_crawler",
    download_url="https://github.com/zldzmfoq12/aud_crawler/archive/0.0.tar.gz",
    keywords=['aud_crawler'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import atexit
from datetime import datetime
import json
from threading import Thread
from urllib.request import Request, urlopen


_format = '%Y-%m-%d %H:%M:%S.%f'
_file = None
_run_name = None
_slack_url = None


def init(filename, run_name, slack_url=None):
    global _file, _run_name, _slack_url
    _close_logfile()
    _file = open(filename, 'a')
    _file.write('\n-----------------------------------------------------------------\n')
    _file.write('Starting new training run\n')
    _file.write('-----------------------------------------------------------------\n')
    _run_name = run_name
    _slack_url = slack_url


def log(msg, slack=False):
    print(msg)
    if _file is not None:
        _file.write('[%s]    %s\n' % (datetime.now().strftime(_format)[:-3], msg))
    if slack and _slack_url is not None:
        Thread(target=_send_slack, args=(msg,)).start()


def _close_logfile():
    global _file
    if _file is not None:
        _file.close()
        _file = None


def _send_slack(msg):
    req = Request(_slack_url)
    req.add_header('Content-Type', 'application/json')
    urlopen(req, json.dumps({
        'username': 'tacotron',
        'icon_emoji': ':taco:',
        'text': '*%s*: %s' % (_run_name, msg)
    }).encode())


atexit.register(_close_logfile)
